#pragma once

#include <string>
#include <iostream>
#include <iomanip>

class TimerCuda
{
public:
    TimerCuda() 
      : _completed(false), _name(""), _elapsed(0.0), _offset(0), _refstart(nullptr), 
        _start(nullptr), _stop(nullptr), _queue(0)
    {
    }

   ~TimerCuda()
    {
        if (_stop)
          cudaEventDestroy(_stop);
        if (_start)
          cudaEventDestroy(_start);
    }

    void init(TimerCuda & Tref,  const char * s = "", int q = 0) {
        _refstart = Tref._start;
        _name = s;
        _queue = q;
    }

    const std::string & name() const { return _name; }

    inline void markStart()
    {
        cudaEventCreate(&_start);
        cudaEventCreate(&_stop);
        cudaEventRecord(_start);
    }
    inline void markStop()
    {
        if (_stop)
          cudaEventRecord(_stop);
    }

    double elapsed() { 
        if (_completed == false && _start && _stop) {
            cudaEventSynchronize(_stop);
            float milliseconds = 0;
            cudaEventElapsedTime(&milliseconds, _start, _stop);
            _elapsed = 1e-3 * milliseconds;
            _completed = true;
        }
        return _elapsed;
    }
    void offset(int n) { _offset = n; };
    int offset() const { return _offset; }
    cudaEvent_t & startEvent() { return _start; }
    cudaEvent_t & stopEvent() { return _stop; }

    void print(std::ostream & f)
    {
      elapsed();
      float milliseconds;
      cudaEventElapsedTime(&milliseconds, _refstart, _start);
      double _startTime = milliseconds*1e-3;
      cudaEventElapsedTime(&milliseconds, _refstart, _stop);
      double _endTime = milliseconds*1e-3;
      f << "stream " << std::setw(3) << _queue << std::setw(20) << std::right << _name << ": " 
        << std::setw(12) << std::setprecision(3) << _elapsed << " s"
        << "\tstart: " 
        << std::setw(12) << std::setprecision(3) << _startTime  << " s"
        << "\tend: " 
        << std::setw(12) << std::setprecision(3) << _endTime  << " s";
    }

private:
    bool _completed;
    std::string _name;
    int _offset;
    double _elapsed;
    cudaEvent_t _start, _stop;
    cudaEvent_t _refstart;
    int _queue;
};

inline std::ostream & operator<<(std::ostream & f, TimerCuda & T)
{
    T.print(f);
    return f;
}

