#pragma once

#include <iostream>
#include <cassert>

#define checkCudaOp(result) __checkCudaOp(result,__FILE__, __LINE__)
inline
cudaError_t __checkCudaOp(cudaError_t result, 
                          const char *file, int line)
{
#if defined(DEBUG) || defined(_DEBUG)
  if (result != cudaSuccess) {
    std::cerr << file << "(" << line << ") : CUDA error: "
              << cudaGetErrorString(result) << std::endl;
    assert(result == cudaSuccess);
  }
#endif
  return result;
}

#define checkCudaKernel() __checkCudaKernel(__FILE__, __LINE__)

inline void __checkCudaKernel(const char *file, int line)
{
#if defined(DEBUG) || defined(_DEBUG)
  cudaDeviceSynchronize();
  cudaError_t err = cudaGetLastError();
  if (err != cudaSuccess) {
    std::cerr << file << "(" << line << ") : CUDA error"
              << cudaGetErrorString(err) << std::endl;
    exit(EXIT_FAILURE);
  }
#endif
}
