#pragma once

#include <cstdio>
#include <cassert>

#define checkStreamKernel(stream) \
  __checkStreamKernel(stream, __FILE__, __LINE__)

inline void __checkStreamKernel(cudaStream_t stream, 
                               const char *file, int line)
{
#if defined(DEBUG) || defined(_DEBUG)
  cudaStreamSynchronize(stream);
  cudaError_t err = cudaGetLastError();
  if (err != cudaSuccess) {
    std::cerr << file << "(" << line << ") : CUDA Error ("
              << cudaGetErrorString(err) << std::endl;
    exit(EXIT_FAILURE);
  }
#endif
}

