#pragma once

#include <string>
#include <iostream>
#include <iomanip>

class TimerCuda
{
public:
    TimerCuda(const char *s = "", int o = 0) 
      : _completed(false), _name(s), _elapsed(0.0), _offset(o)
    {
        cudaEventCreate(&_start);
        cudaEventCreate(&_stop);
    }

    ~TimerCuda()
    {
        cudaEventDestroy(_stop);
        cudaEventDestroy(_start);
    }

    const std::string & name() const { return _name; }
    void name(const char *newName) { _name = newName; }
    inline void markStart()
    {
        cudaEventRecord(_start);
    }
    inline void markStop()
    {
        cudaEventRecord(_stop);
    }

    double elapsed() { 
        if (_completed == false) {
            cudaEventSynchronize(_stop);
            float milliseconds = 0;
            cudaEventElapsedTime(&milliseconds, _start, _stop);
            _elapsed = 1e-3 * milliseconds;
            _completed = true;
        }
        return _elapsed;
    }
    void offset(int n) { _offset = n; };
    int offset() const { return _offset; }

private:
    bool _completed;
    std::string _name;
    int _offset;
    double _elapsed;
    cudaEvent_t _start, _stop;
  
};

inline std::ostream & operator<<(std::ostream & f, TimerCuda & T)
{
    f << std::setw(20) << std::right << T.name() << " : " << T.elapsed() << " s" << std::endl;
    return f;
}
